@@ -307,6 +307,7 @@ module Schleuder
       new.to = recv.email.to_s
       new.date = Time.now
       new.from = Schleuder.list.config.myaddr
+      new = _add_openpgp_header(new) if Schleuder.list.config.include_openpgp_header
       if self.multipart?
         self.parts.each do |p|
           new.parts.push(Mail.parse(p.to_s))
@@ -319,12 +320,8 @@ module Schleuder
 
     def individualize_member(recv)
       new = self.individualize(recv)
-      if Schleuder.list.config.keep_msgid
-        Schleuder.log.debug "Copying msgid to in-reply-to/references"
-        new.in_reply_to = _collect_message_ids(self.in_reply_to)
-        new.references = _collect_message_ids(self.references)
-      end
-      new
+      _message_ids(new) if Schleuder.list.config.keep_msgid
+      _list_headers(new) if Schleuder.list.config.include_list_headers
     end
 
     # Strips keywords from self and stores them in @+keywords+
@@ -399,21 +396,27 @@ module Schleuder
     end
 
     def add_prefix!
-      Schleuder.log.debug "adding prefix"
       # only add if it's not already present
       unless self.subject.index(Schleuder.list.config.prefix)
+        Schleuder.log.debug "adding prefix"
         self.subject = Schleuder.list.config.prefix + " " + self.subject.to_s
       end
     end
 
     def add_prefix_in!
-      Schleuder.log.debug "adding prefix_in"
-      self.subject = Schleuder.list.config.prefix_in + " " + self.subject.to_s
+      # only add if it's not already present
+      unless self.subject.index(Schleuder.list.config.prefix_in)
+        Schleuder.log.debug "adding prefix_in"
+        self.subject = Schleuder.list.config.prefix_in + " " + self.subject.to_s
+      end
     end
 
     def add_prefix_out!
-      Schleuder.log.debug "adding prefix_out"
-      self.subject = Schleuder.list.config.prefix_out + " " + self.subject.to_s
+      # only add if it's not already present
+      unless self.subject.index(Schleuder.list.config.prefix_out)
+        Schleuder.log.debug "adding prefix_out"
+        self.subject = Schleuder.list.config.prefix_out + " " + self.subject.to_s
+      end
     end
 
     def add_metadata!
@@ -638,8 +641,8 @@ module Schleuder
       p = Mail.new
       p.body = body.to_s
       p.content_type = content_type
-      p.charset = charset.to_s if charset
-      p.encoding = encoding.to_s if encoding
+      p.charset = charset.to_s unless charset.to_s.empty?
+      p.encoding = encoding.to_s unless encoding.to_s.empty?
       p
     end
 
@@ -652,5 +655,60 @@ module Schleuder
       @@schleuder_message_id = Utils.generate_message_id(Schleuder.list.listname) unless @@schleuder_message_id
       @@schleuder_message_id
     end
+
+    def _message_ids(mail)
+      Schleuder.log.debug "Copying msgid to in-reply-to/references"
+      mail.in_reply_to = _collect_message_ids(self.in_reply_to)
+      mail.references = _collect_message_ids(self.references)
+      mail
+    end
+
+    def _list_headers(mail)
+      Schleuder.log.debug "Generating list-ids"
+      mail['List-Id'] = _list_id
+      mail
+    end
+
+    def _list_id
+      "<#{Schleuder.list.config.myaddr.gsub(/@/,'.')}>"
+    end
+
+    def _add_openpgp_header(mail)
+      Schleuder.log.debug "Add OpenPGP-Headers"
+      mail['OpenPGP'] = "id=#{@crypt.get_key(Schleuder.list.config.myaddr).subkeys.first.keyid} "+
+                        "(Send an email to #{Schleuder.list.sendkey_addr} to receive the public-key)"+
+                        _gen_openpgp_pref_header
+      mail
+    end
+
+    def _gen_openpgp_pref_header
+      unless Schleuder.list.config.openpgp_header_preference == 'none'
+        pref_str = "; preference=#{Schleuder.list.config.openpgp_header_preference} ("
+        if !Schleuder.list.config.receive_authenticated_only
+          if Schleuder.list.config.receive_encrypted_only \
+              && Schleuder.list.config.receive_signed_only
+            pref_str << 'Only encrypted and signed emails are accepted'
+          elsif Schleuder.list.config.receive_encrypted_only \
+                  && !Schleuder.list.config.receive_signed_only
+            pref_str << 'Only encrypted emails are accepted'
+          elsif !Schleuder.list.config.receive_encrypted_only \
+                  && Schleuder.list.config.receive_signed_only
+            pref_str << 'Only signed emails are accepted'
+          else
+            pref_str << 'All kind of emails are accepted'
+          end
+        elsif Schleuder.list.config.receive_authenticated_only
+          if Schleuder.list.config.receive_encrypted_only
+            pref_str << 'Only encrypted and signed emails by list-members are accepted'
+          else
+            pref_str << 'Only signed emails by list-members are accepted'
+          end
+        else
+          pref_str << 'All kind of emails are accepted'
+        end
+        pref_str << ')'
+      end
+      pref_str || ''
+    end
   end
 end
